	FUNCTION cyl_gauss(a,z)

C Date written:	August 1, 1996
C Revisions:
C Author:	Thomas Bortfeld, DKFZ Heidelberg
C Keywords:	special functions, parabolic cylinder function,
C		degenerate hypergeometric function
C Description:	This is a straight-forward implementation of the product of
C		the parabolic cylinder function D_a(z) with exp(-z**2/4),
C		which is valid for real arguments and negative a, 0 > a > -2.
C		It uses a Taylor series for z close to 0, (cf. [1], sections
C		9.240 and 9.210), and an asymptotic expansion for larger z
C		(cf. [1], section 9.246). 
C References:	[1] Gradshteyn & Ryzhik; Table of integrals, series and 
C		products; Academic Press; London; 1980.	 

	IMPLICIT NONE

	REAL cyl_gauss, a, z, pi, hyperg, gamma_ln, term1, term2, c1, c2
	REAL z_square, term_i, asympt, asympt_2, u, branch
	INTEGER i, nterm, nterm_2
	PARAMETER (branch = 4.0, nterm = 6)
	PARAMETER (pi = 3.14159265)

	if (a.ge.0.0 .or. a.lt.-2.0) stop 'A is out of range'

	if (abs(z).gt.branch) then
C					use asymptotic expansion
	  z_square = z*z
	  nterm_2 = int(nterm*branch/abs(z)+0.5)
ccc	  type *, nterm_2

	  asympt = 1.
	  term_i = 1.
	  u = a
	  do i = 1, nterm_2
	    term_i = term_i * u * (1.-u) / (2.*i*z_square)
ccc	    type *, term_i
	    asympt = asympt + term_i
	    u = u - 2.
	  enddo

	  if (z.gt.0.) then
	    cyl_gauss = exp(-z_square/2.) * z**a * asympt
	  else
C					asymptotic expansion for z < 0
	    asympt_2 = 1.
	    term_i = 1.
	    u = a + 1
	    do i = 1, nterm_2
	      term_i = term_i * u * (u+1.) / (2.*i*z_square)
ccc	      type *, term_i
	      asympt_2 = asympt_2 + term_i
	      u = u + 2.
	    enddo
	    cyl_gauss = cos(pi*a) * exp(-z_square/2.) * (-z)**a * asympt
	1	+ sqrt(2.*pi) / exp(gamma_ln(-a)) * (-z)**(-a-1.) * asympt_2
	  endif

	else
C					use Taylor series
	  c1 = 0.5
	  c2 = 1.5
	  term1 = sqrt(pi)/exp(gamma_ln((1.-a)/2.)) 
	1		* hyperg(c1+a/2.,c1,-z**2/2.)
	  term2 = sqrt(2.*pi)*z/exp(gamma_ln(-a/2.)) 
	1		* hyperg(c2+(a-1.)/2.,c2,-z**2/2.)
ccc	  type *, 'term1:', term1, '       term2:', term2
	  cyl_gauss = 2.**(a/2.) * (term1-term2)

	endif

	return
	END



	FUNCTION hyperg(a,b,z)

C Date written:	August 1, 1996
C Revisions:
C Author:	Thomas Bortfeld, DKFZ Heidelberg
C Keywords:	special functions, degenerate hypergeometric function
C Description:	Taylor series expansion of the degenerate hypergeometric
C		function Phi(alpha,gamma,z) (cf. [1], section 
C		9.210), useful for small values of |z|. The value of eps
C		can be adjusted to calculate more or less significant 
C		digits.
C References:	[1] Gradshteyn & Ryzhik; Table of integrals, series and 
C		products; Academic Press; London; 1980.	 

	IMPLICIT NONE

	REAL hyperg, a, b, z, term_i, eps
	INTEGER i
	PARAMETER (eps = 0.00001)

	hyperg = 1.
	term_i = 1.

	do i = 1, 1000
	  term_i = term_i * (a+i-1.)*z/(b+i-1.)/float(i)
	  hyperg = hyperg + term_i
	  if(abs(term_i) .lt. eps*hyperg) then
ccc	    type *, 'Number of iterations:', i
	    return
	  endif
	enddo

	pause 'No convergence in HYPERG !!!'

	return
	END



	FUNCTION gamma_ln(xx)

C Date written:
C Revisions:
C Author:
C Keywords:	special functions, gamma function
C Description:	Calculates the logarithm of the gamma function for positive
C		arguments according to [1].
C References:	[1] Press, Teukolsky, Vetterling & Flannery; 
C		Numerical recipes; 2nd. ed.; Cambridge University Press; 1992.

	IMPLICIT NONE

	REAL gamma_ln, xx
	INTEGER j
	DOUBLE PRECISION ser, stp, tmp, x, y, cof(6)

	save cof, stp
	data cof, stp/76.18009172947146d0,-86.50532032941677d0,
	1  24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
	1  -.5395239384953d-5,2.5066282746310005d0/

	x = xx
	y = x
	tmp = x + 5.5d0
	tmp = (x+0.5d0)*log(tmp) - tmp
	ser = 1.000000000190015d0

	do j = 1, 6
	  y = y + 1.d0
	  ser = ser + cof(j)/y
	enddo
	gamma_ln = tmp + log(stp*ser/x)

	return
	END
