function out=cyl_gauss(a,z);

% Date written:	November 13, 2001
% Revisions:
% Author:	Thomas Bortfeld, MGH Boston
% Keywords:	special functions, parabolic cylinder function,
%		degenerate hypergeometric function
% Description:	This is a straight-forward implementation of the product of
%		the parabolic cylinder function D_a(z) with exp(-z**2/4),
%		which is valid for real arguments and negative a, 0 > a > -2.
%		It uses a Taylor series for z close to 0, (cf. [1], sections
%		9.240 and 9.210), and an asymptotic expansion for larger z
%		(cf. [1], section 9.246). 
% References:	[1] Gradshteyn & Ryzhik; Table of integrals, series and 
%		products; Academic Press; London; 1980.	 


branch = 4.;
nterm = 6;

if ((a >= 0.) | (a <= -2.))
    error('parameter a is out of range');
end;

if (abs(z) >= branch)
    
    z_square = z*z;
    nterm_2 = round(nterm*branch/abs(z));

    asympt = 1.;
    term_i = 1.;
    u = a;
    
    for i=1:nterm_2
        term_i = term_i * u * (1.-u) / (2.*i*z_square);
        asympt = asympt + term_i;
        u = u - 2.;
    end

    if (z >= 0.)
        out = exp(-z_square/2.) * z^a * asympt;
    else        % asymptotic expansion for z < 0
        asympt_2 = 1.;
	    term_i = 1.;
	    u = a + 1;

        for i=1:nterm_2
            term_i = term_i * u * (u+1.) / (2.*i*z_square);
            asympt_2 = asympt_2 + term_i;
            u = u + 2.;
        end
        
	    out = cos(pi*a) * exp(-z_square/2.) * (-z)^a * asympt + sqrt(2.*pi) / gamma(-a) * (-z)^(-a-1.) * asympt_2;
    end
    
else    % use Taylor series
    c1 = 0.5;
    c2 = 1.5;
    term1 = sqrt(pi)/gamma((1.-a)/2.) * hyperg(c1+a/2.,c1,-z^2/2.);
    term2 = sqrt(2.*pi)*z/gamma(-a/2.) * hyperg(c2+(a-1.)/2.,c2,-z^2/2.);
    out = 2.^(a/2.) * (term1-term2);
end
